"""PYNTeractA

Usage:
------

    $ pynta

Contact:
--------

- https://www.unionerenolavinosamoggia.bo.it/index.php/contatti

More information is available at:

- https://github.com/simodalla/pynteracta


Version:
--------

- pynteracta v0.4.12
"""

from .cli.commands import app

if __name__ == "__main__":
    app()
